#pragma once

#include "wrap/thread.h"
#include "wrap/synch.h"
#include "wrap/atomics.h"
#include <stdio.h>
#include <stdlib.h>

/**
 * This file contains common code that is included by the different implementations. Make sure the
 * file is only included once for each binary, otherwise functions will be defined multiple times.
 *
 * Defines 'main' and the test program. Expects to find the functions declared below implemented in
 * the c-file that included this header.
 */

// Called once to initialize any data.
void pinit(void);

// Lock and unlock.
void plock(int tid);
void punlock(int tid);


// Shared data protected by the lock implemented by 'plock' and 'punlock'.
int seq = 0;

static void seq_0(void *input) {
	int tid = 0;

	for ( ; ; ) {
		plock(tid);
		fprintf(stdout, "%d Peterson was very clever.\n", ++seq);
		punlock(tid);
	}

	sema_up(input);
}

static void seq_1(void *input) {
	int tid = 1;

	for ( ; ; ) {
		plock(tid);
		fprintf(stdout, "%d The solution may actually work!\n", ++seq);
		punlock(tid);
	}

	sema_up(input);
}

int main(void) {
	struct semaphore zero, one;

	pinit();
	sema_init(&zero, 0);
	sema_init(&one, 0);

	thread_create("", 0, &seq_0, &zero);
	thread_create("", 1, &seq_1, &one);

	sema_down(&zero);
	sema_down(&one);

	return 0;
}

